#!/usr/bin/env python

###
###
###

import collections
import math

from PyQt5 import QtWidgets

import gui.canvas

Coords = collections.namedtuple('Coords', 'x y')

class Scene(QtWidgets.QGraphicsScene):
    def __init__(self):
        super().__init__()
        self.setItemIndexMethod(QtWidgets.QGraphicsScene.NoIndex)
        self.changed.connect(self.resize)

    # co to dela?!
    # asi zmena celkoveho obdelniku, pokud se zmeni neco ve scene
    # -> zajistit, aby pri posouvani vzdy byl padding 20 na kazdou stranu
    def resize(self):
        self.setSceneRect(self.itemsBoundingRect().adjusted(-20, -20, 20, 20))

    def create_datapath(self, ip, dpid, level):
        return gui.canvas.Datapath(ip, dpid, level)

    def create_node(self, ip, mac, parent_port):
        return gui.canvas.Node(ip, mac, parent_port)

    def create_link(self, element1, element2):
        link = gui.canvas.Link(element1, element2)
        element1.links.add(link)
        element2.links.add(link)
        return link

    def add_element(self, element, pos):
        x = pos[0]
        y = pos[1]

        #if element.scene() != self:
        self.addItem(element)
        element.setPos(x, y)

    def add_link(self, link):
        #if link.scene() != self:
        self.addItem(link)

        stats = gui.canvas.Stats(link.source, link.destination)
        link.attach_stats(stats)
        self.addItem(stats)

        link.draw()

    def remove_element(self, element):
        self.removeItem(element)

    def remove_link(self, link):
        self.removeItem(link.stats)
        self.removeItem(link)


class SceneMatrix:
    _field_size = 25

    @classmethod
    def pos_to_scene(cls, pos):
        return Coords(pos.x*cls._field_size, pos.y*cls._field_size)

    @classmethod
    def element_matrix_size(cls, element):
        rect = element.boundingRect().size()
        width = int(math.ceil(rect.width() / cls._field_size))
        height = int(math.ceil(rect.height() / cls._field_size))
        return (width, height)
